@@ -1,9 +1,12 @@
 summary.meta <- function(object,
-                         byvar,
-                         bylab,
+                         byvar=object$byvar,
+                         bylab=object$bylab,
+                         print.byvar=object$print.byvar,
                          bystud=FALSE,
                          level=object$level,
                          level.comb=object$level.comb,
+                         comb.fixed=object$comb.fixed,
+                         comb.random=object$comb.random,
                          warn=TRUE,
                          ...){
   
@@ -25,17 +28,31 @@ summary.meta <- function(object,
   Q  <- object$Q
   
   
+  if (length(comb.fixed)==0){
+    comb.fixed <- TRUE
+  }
+  ##
+  if (length(comb.random)==0){
+    comb.random <- TRUE
+  }
+  ##
+  if (length(print.byvar)==0){
+    print.byvar <- TRUE
+  }
+  
+  
   if (length(level)==0){
     warning("level set to 0.95")
     level <- 0.95
   }
   ##
   if (length(level.comb)==0){
-    ##if (comb.fixed | comb.random)
-    warning("level.comb set to 0.95")
+    if (comb.fixed | comb.random)
+      warning("level.comb set to 0.95")
     level.comb <- 0.95
   }
   
+  
   ##
   ## Higgins & Thompson (2002), Statistics in Medicine, 21, 1539-58
   ##
@@ -93,24 +110,21 @@ summary.meta <- function(object,
   }
   
   
-  if (!missing(byvar)){
-
-    byvar.name <- deparse(substitute(byvar))
+  if (!missing(byvar) & length(object$byvar)==0){
+    byvar.name <- deparse(substitute(byvar))  
     if (!is.null(object[[byvar.name]]))
       byvar <- object[[byvar.name]]
-    
+  }
+  
+  
+  if (length(byvar)>0){
     
     if (any(is.na(byvar))) stop("Missing values in 'byvar'")
-
-    print(object$method)
-    if (object$method == "MH")
-      warning("Test for subgroup differences may be invalid for method=\"MH\"")
-
+    
     by.levs <- unique(byvar)
     
-    if (missing(bylab)) bylab <- byvar.name
+    if (length(bylab)==0) bylab <- deparse(substitute(byvar))
     
-
     res.w <- matrix(NA, ncol=4, nrow=length(by.levs))
     j <- 0
     ##
@@ -118,11 +132,15 @@ summary.meta <- function(object,
       j <- j+1
       sel <- byvar == i
       ##
+      if (all(is.na(object$studlab[sel])))
+        stop("No data available for byvar = ", i)
+      ##
+      ##
       if (inherits(object, "metabin")){
         meta1 <- metabin(object$event.e[sel], object$n.e[sel],
                          object$event.c[sel], object$n.c[sel],
                          studlab=object$studlab[sel],
-                         method="Inverse",
+                         method=object$method,
                          sm=object$sm,
                          incr=object$incr,
                          allincr=object$allincr,
@@ -130,8 +148,8 @@ summary.meta <- function(object,
                          allstudies=object$allstudies,
                          MH.exact=object$MH.exact,
                          RR.cochrane=object$RR.cochrane,
-                         level=object$level,
-                         level.comb=object$level.comb,
+                         level=level, level.comb=level.comb,
+                         comb.fixed=comb.fixed, comb.random=comb.random,
                          warn=object$warn)
       }
       ##
@@ -140,27 +158,31 @@ summary.meta <- function(object,
                           object$n.c[sel], object$mean.c[sel], object$sd.c[sel],
                           sm=sm,
                           studlab=object$studlab[sel],
-                          level=object$level,
-                          level.comb=object$level.comb)
+                          level=level, level.comb=level.comb,
+                          comb.fixed=comb.fixed, comb.random=comb.random)
       }
       ##
       if (inherits(object, "metagen")){
         meta1 <- metagen(object$TE[sel], object$seTE[sel], sm=sm,
                          studlab=object$studlab[sel],
-                         level=object$level,
-                         level.comb=object$level.comb)
+                         level=level, level.comb=level.comb,
+                         comb.fixed=comb.fixed, comb.random=comb.random)
       }
       ##
       if (inherits(object, "metaprop")){
         meta1 <- metaprop(object$event[sel], object$n[sel],
                           studlab=object$studlab[sel],
                           freeman.tukey=object$freeman.tukey,
-                          level=object$level,
-                          level.comb=object$level.comb)
+                          level=level, level.comb=level.comb,
+                          comb.fixed=comb.fixed, comb.random=comb.random)
       }
       ##
       if (bystud){
-        bylab2 <- paste(bylab, " = ", i, sep="")
+        if (print.byvar)
+          bylab2 <- paste(bylab, " = ", i, sep="")
+        else
+          bylab2 <- i
+        ##
         lab <- paste(rep("-", nchar(bylab2)), collapse="")
         ##
         cat(lab, "\n", sep="")
@@ -170,8 +192,14 @@ summary.meta <- function(object,
         print(meta1, details=FALSE, ma=FALSE)
       }
       ##
-      res.w[j,] <- c(meta1$TE.fixed, meta1$seTE.fixed,
-                     meta1$Q, meta1$k)
+      if (comb.fixed)
+        res.w[j,] <- c(meta1$TE.fixed, meta1$seTE.fixed,
+                       meta1$Q, meta1$k)
+      else if (comb.random&!comb.fixed)
+        res.w[j,] <- c(meta1$TE.random, meta1$seTE.random,
+                       meta1$Q, meta1$k)
+      else
+        res.w[j,] <- c(NA, NA, NA, NA)
     }
     ##
     TE.fixed.w <- res.w[,1]
@@ -184,10 +212,15 @@ summary.meta <- function(object,
     Q.b <- sum(1/seTE.fixed.w^2*(TE.fixed.w - object$TE.fixed)^2,
                na.rm=TRUE)
     ##
-    if ((round(Q-sum(Q.w, na.rm=TRUE),2) - round(Q.b,2)) != 0)
-      warning(paste("Q-sum(Q.w) != Q.b\nQ.b =", round(Q.b,2)))
+    if (object$method != "MH" & (comb.fixed|comb.random)){
+      if ((round(Q-sum(Q.w, na.rm=TRUE),2) - round(Q.b,2)) != 0)
+        warning(paste("Q-sum(Q.w) != Q.b\nQ.b =", round(Q.b,2)))
+    }
     ##
     if (bystud) cat("\n")
+    ##
+    if (object$method=="MH" | !comb.fixed)
+      Q.w <- rep(NA, length(Q.w))
   }
 
 
@@ -198,10 +231,12 @@ summary.meta <- function(object,
               Q.CMH=object$Q.CMH,
               sm=sm, method=object$method,
               call=match.call(),
-              ci.lab=ci.lab)
+              ci.lab=ci.lab,
+              comb.fixed=comb.fixed,
+              comb.random=comb.random)
 
   
-  if (!missing(byvar)){
+  if (length(byvar)>0){
     res$within  <- ci.w
     res$k.w     <- k.w
     res$Q.w     <- Q.w
@@ -226,6 +261,11 @@ summary.meta <- function(object,
     class(res) <- c(class(res), "trimfill")
   }
   
+  res$complab <- object$complab
+  res$outclab <- object$outclab
+  res$title   <- object$title
+  ##
+  res$print.byvar <- print.byvar
   
   res
 }
