@@ -1,7 +1,8 @@
 trimfill <- function(x, seTE, left=NULL, ma.fixed=TRUE,
                      type="L", n.iter.max=50,
                      sm=NULL, studlab=NULL,
-                     level, level.comb,
+                     level=x$level, level.comb=x$level.comb,
+                     comb.fixed=x$comb.fixed, comb.random=x$comb.random,
                      silent=TRUE){
   
   
@@ -11,17 +12,25 @@ trimfill <- function(x, seTE, left=NULL, ma.fixed=TRUE,
     stop("This function is not usable for an object of class \"metainf\"")
   
   
-  if (missing(level))
-    if (! "level" %in% names(x))
-      level <- 0.95
-    else
-      level <- x$level
+  if (length(comb.fixed)==0){
+    comb.fixed <- TRUE
+  }
   ##
-  if (missing(level.comb))
-    if (! "level.comb" %in% names(x))
-      level.comb <- 0.95
-    else
-      level.comb <- x$level.comb
+  if (length(comb.random)==0){
+    comb.random <- TRUE
+  }
+  
+  
+  if (length(level)==0){
+    warning("level set to 0.95")
+    level <- 0.95
+  }
+  ##
+  if (length(level.comb)==0){
+    if (comb.fixed | comb.random)
+      warning("level.comb set to 0.95")
+    level.comb <- 0.95
+  }
   
   
   estimate.missing <- function(TE, seTE, TE.sum, type){
@@ -203,7 +212,8 @@ trimfill <- function(x, seTE, left=NULL, ma.fixed=TRUE,
               n.iter=n.iter,
               trimfill=trimfill,
               k0=sum(trimfill),
-              level=level, level.comb=level.comb)
+              level=level, level.comb=level.comb,
+              comb.fixed=comb.fixed, comb.random=comb.random)
   ##
   class(res) <- c("metagen", "meta", "trimfill")
   ##
